{***************************************************************
 *
 * Unit Name : DataAdjustDlg
 * Purpose   : Database Smart Compression
 * Author    : Eugen Mihailescu
 * Company   : EM Quicksoft Romania SRL
 * Copyright : 1998,2002  All rights reserved.
 * Web Site  : http://www.em-quicksoft.com
 * Support   : support@em-quicksoft.com
 * History   : 03/17/2000
 *
 * Disclaimer: The objectives of this program is only educational.
 *             You MUST NOT use it without getting any material benefit on it
 *             You have the right to share this code with others without removing
 *             the copyright and/or disclaimer notice.
 *             You are allowed to include its code in your application
 *             only if it was designed for non commercial purpose,
 *             otherwise we claim $10 for each copy of your
 *             application which contains this code inside.
 ****************************************************************}

Unit DataAdjustDlg;

Interface

Uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Db,
  DBTables,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  Spin,
  FileCtrl,
  Mask,
  Math,
  IniFiles;

Type
  TForm1 = Class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    L1: TSpinEdit;
    L2: TSpinEdit;
    L4: TSpinEdit;
    L3: TSpinEdit;
    L8: TSpinEdit;
    L7: TSpinEdit;
    L6: TSpinEdit;
    L5: TSpinEdit;
    L12: TSpinEdit;
    L11: TSpinEdit;
    L10: TSpinEdit;
    L9: TSpinEdit;
    L13: TSpinEdit;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    btnStop: TButton;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    StatusBar1: TStatusBar;
    ProgressBar1: TProgressBar;
    Bevel1: TBevel;
    btnCreaza: TButton;
    btnTerminare: TButton;
    Button4: TButton;
    Button5: TButton;
    SaveDialog1: TSaveDialog;
    Table1: TTable;
    btnCalc: TButton;
    MaskEdit1: TMaskEdit;
    MaskEdit2: TMaskEdit;
    Query1: TQuery;
    Table2: TTable;
    Button1: TButton;
    Procedure FormCreate(Sender: TObject);
    Procedure Button4Click(Sender: TObject);
    Procedure Button5Click(Sender: TObject);
    Procedure btnCreazaClick(Sender: TObject);
    Procedure btnStopClick(Sender: TObject);
    Procedure FormDestroy(Sender: TObject);
    Procedure btnTerminareClick(Sender: TObject);
    Procedure btnCalcClick(Sender: TObject);
    Procedure FormClose(Sender: TObject; Var Action: TCloseAction);
    Procedure Button1Click(Sender: TObject);
  private
    Aborted: Boolean;
    LstFisiere: TStringList;
    Procedure EnableControls(Value: Boolean);
    Procedure CreazaLista;
    Procedure Calculeaza;
    Function GetAllTableRecCount: Integer;
    Procedure SetMax(S: String; Index: Integer);
    Procedure SaveSettings;
    Procedure LoadSettings;
    Procedure CreazaAlfabet;
    Function ReplaceDiacritice(Value: String): String;
    Procedure ReplaceAllDiacritics;
  public
  End;

Var
  Form1             : TForm1;

Implementation

{$R *.DFM}
//******************* SlashSep *************************

Function SlashSep(Const Path, S: String): String;
Begin
  If AnsiLastChar(Path)^ <> '\' Then
    Result := Path + '\' + S
  Else
    Result := Path + S;
End;
//******************* TForm1.FormCreate *************************

Procedure TForm1.FormCreate(Sender: TObject);
Begin
  LstFisiere := TStringList.Create;
  ProgressBar1.Parent := StatusBar1;
  ProgressBar1.Left := 1;
  ProgressBar1.Top := 2;
  ProgressBar1.Width := 198;
  EnableControls(True);
  LoadSettings;
End;
//******************* TForm1.Button4Click *************************

Procedure TForm1.Button4Click(Sender: TObject);
Var
  Dir               : String;
Begin
  If SelectDirectory('Selectati folderul ce contine fisierele', '', Dir) Then
    Edit1.Text := Dir;
End;
//******************* TForm1.Button5Click *************************

Procedure TForm1.Button5Click(Sender: TObject);
Begin
  SaveDialog1.FileName := Edit3.Text;
  If SaveDialog1.Execute Then
    Edit3.Text := SaveDialog1.FileName;
End;
//******************* TForm1.EnableControls *************************

Procedure TForm1.EnableControls(Value: Boolean);
Var
  I                 : Integer;
Begin
  For I := 0 To ControlCount - 1 Do
    Controls[I].Enabled := Not (Value Xor (Controls[I] <> btnStop));
End;
//******************* TForm1.btnCreazaClick *************************

Procedure TForm1.btnCreazaClick(Sender: TObject);
Begin
  CreazaLista;
  CreazaAlfabet;
End;
//******************* TForm1.btnStopClick *************************

Procedure TForm1.btnStopClick(Sender: TObject);
Begin
  Aborted := MessageDlg('Sunteti sigur ca doriti sa opriti procesul curent?', mtConfirmation, [mbYes, mbNo], 0) = mrYes;
End;
//******************* TForm1.CreazaLista *************************

Procedure TForm1.CreazaLista;
Var
  R                 : TSearchRec;
  Found             : Integer;
Begin
  Found := FindFirst(SlashSep(Edit1.Text, '') + Edit2.Text + '*.DB', faAnyFile, R);
  While Found = 0 Do
    Begin
      LstFisiere.Add(SlashSep(Edit1.Text, '') + R.Name);
      Found := FindNext(R);
    End;
  FindClose(R);
End;
//******************* TForm1.FormDestroy *************************

Procedure TForm1.FormDestroy(Sender: TObject);
Begin
  LstFisiere.Free;
End;
//******************* TForm1.Calculeaza *************************

Procedure TForm1.Calculeaza;
Var
  I, j, k, Size     : Integer;
Begin
  ProgressBar1.Max := GetAllTableRecCount;
  ProgressBar1.Visible := True;
  EnableControls(False);
  For I := 0 To LstFisiere.Count - 1 Do
    Begin
      If Aborted Then Break;
      Table1.Active := False;
      Table1.TableName := LstFisiere.Strings[I];
      Table1.Active := True;
      StatusBar1.Panels[1].Text := Format('Procesez fisierul %s', [LstFisiere.Strings[I]]);
      k := 0;
      Size := Table1.RecordCount;
      While Not (Table1.Eof Or Aborted) Do
        Begin
          For j := 0 To Table1.FieldDefs.Count - 1 Do
            SetMax(Table1.Fields[j].AsString, j);
          Table1.Next;
          ProgressBar1.StepIt;
          Inc(k);
          StatusBar1.Panels[2].Text := Format('%d / %d', [k, Size]);
          Application.ProcessMessages;
        End;
    End;
  EnableControls(True);
  ProgressBar1.Visible := False;
End;
//******************* TForm1.GetAllTableRecCount *************************

Function TForm1.GetAllTableRecCount: Integer;
Var
  I                 : Integer;
Begin
  Result := 0;
  For I := 0 To LstFisiere.Count - 1 Do
    Begin
      Table1.Active := False;
      Table1.TableName := LstFisiere.Strings[I];
      Table1.Active := True;
      Result := Result + Table1.RecordCount;
    End;
End;
//******************* TForm1.SetMax *************************

Procedure TForm1.SetMax(S: String; Index: Integer);
Var
  I                 : Integer;
  SpinEdit          : TSpinEdit;
Begin
  For I := 0 To ControlCount - 1 Do
    If (Controls[I].Tag = Index) And (Controls[I] Is TSpinEdit) Then
      Begin
        SpinEdit := Controls[I] As TSpinEdit;
        Break;
      End;
  If SpinEdit = Nil Then Exit;
  SpinEdit.Value := Max(SpinEdit.Value, Length(S));
End;
//******************* TForm1.btnTerminareClick *************************

Procedure TForm1.btnTerminareClick(Sender: TObject);
Begin
  Close;
End;
//******************* TForm1.btnCalcClick *************************

Procedure TForm1.btnCalcClick(Sender: TObject);
Begin
  CreazaLista;
  Calculeaza;
End;

Procedure TForm1.SaveSettings;
Var
  F                 : TIniFile;
Begin
  F := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'Settings.INI');
  Try
    F.WriteString('Path', 'FilePath', Edit1.Text);
    F.WriteString('Path', 'FilePrefix', Edit2.Text);
    F.WriteString('Path', 'NewFileName', Edit3.Text);
    F.WriteString('Char', 'From', MaskEdit1.Text);
    F.WriteString('Char', 'To', MaskEdit2.Text);
    F.WriteInteger('Length', '1', L1.Value);
    F.WriteInteger('Length', '2', L2.Value);
    F.WriteInteger('Length', '3', L3.Value);
    F.WriteInteger('Length', '4', L4.Value);
    F.WriteInteger('Length', '5', L5.Value);
    F.WriteInteger('Length', '6', L6.Value);
    F.WriteInteger('Length', '7', L7.Value);
    F.WriteInteger('Length', '8', L8.Value);
    F.WriteInteger('Length', '9', L9.Value);
    F.WriteInteger('Length', '10', L10.Value);
    F.WriteInteger('Length', '11', L11.Value);
    F.WriteInteger('Length', '12', L12.Value);
    F.WriteInteger('Length', '13', L13.Value);
  Finally
    F.Free;
  End;
End;
//******************* TForm1.LoadSettings *************************

Procedure TForm1.LoadSettings;
Var
  F                 : TIniFile;
Begin
  F := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'Settings.INI');
  Try
    Edit1.Text := F.ReadString('Path', 'FilePath', '');
    Edit2.Text := F.ReadString('Path', 'FilePrefix', '');
    Edit3.Text := F.ReadString('Path', 'NewFileName', '');
    MaskEdit1.Text := F.ReadString('Char', 'From', 'A');
    MaskEdit2.Text := F.ReadString('Char', 'To', 'A');
    L1.Value := F.ReadInteger('Length', '1', 0);
    L2.Value := F.ReadInteger('Length', '2', 0);
    L3.Value := F.ReadInteger('Length', '3', 0);
    L4.Value := F.ReadInteger('Length', '4', 0);
    L5.Value := F.ReadInteger('Length', '5', 0);
    L6.Value := F.ReadInteger('Length', '6', 0);
    L7.Value := F.ReadInteger('Length', '7', 0);
    L8.Value := F.ReadInteger('Length', '8', 0);
    L9.Value := F.ReadInteger('Length', '9', 0);
    L10.Value := F.ReadInteger('Length', '10', 0);
    L11.Value := F.ReadInteger('Length', '11', 0);
    L12.Value := F.ReadInteger('Length', '12', 0);
    L13.Value := F.ReadInteger('Length', '13', 0);
  Finally
    F.Free;
  End;
End;
//******************* TForm1.FormClose *************************

Procedure TForm1.FormClose(Sender: TObject; Var Action: TCloseAction);
Begin
  SaveSettings;
End;
//******************* TForm1.CreazaAlfabet *************************

Procedure TForm1.CreazaAlfabet;
Var
  I, j, k, Size     : Integer;
  S                 : String;
Begin
  ProgressBar1.Max := GetAllTableRecCount;
  ProgressBar1.Visible := True;
  EnableControls(False);
  If LstFisiere.Count > 0 Then
    Begin
      Table1.Active := False;
      Table1.TableName := LstFisiere.Strings[0];
      Table1.Active := True;
      Table2.Active := False;
      Table2.TableName := Edit3.Text;
      Table2.FieldDefs.Clear;
      For k := 0 To Table1.FieldDefs.Count - 1 Do
        Table2.FieldDefs.Add(Table1.FieldDefs.Items[k].Name, Table1.FieldDefs.Items[k].DataType, Table1.FieldDefs.Items[k].Size, False);
      Table1.Active := False;
      Table2.CreateTable;
    End;

  For I := 0 To LstFisiere.Count - 1 Do
    Begin
      If Aborted Then Break;
      StatusBar1.Panels[1].Text := Format('Procesez fisierul %s', [LstFisiere.Strings[I]]);
      Query1.Active := False;
      Query1.SQL.Text := Format('INSERT INTO "%s" SELECT * FROM "%s" WHERE (SUBSTRING(NUME FROM 1 FOR 1)>="%s" AND SUBSTRING(NUME FROM 1 FOR 1)<="%s")', [Edit3.Text, LstFisiere.Strings[I], MaskEdit1.Text, MaskEdit2.Text]);
      Query1.Active := True;
      Application.ProcessMessages;
    End;
  EnableControls(True);
  ProgressBar1.Visible := False;
End;
//******************* TForm1.ReplaceDiacritice *************************

Function TForm1.ReplaceDiacritice(Value: String): String;
Begin
  Value := StringReplace(Value, char(195), 'A', [rfReplaceAll]);
  Value := StringReplace(Value, char(194), 'A', [rfReplaceAll]);
  Value := StringReplace(Value, char(170), 'S', [rfReplaceAll]);
  Value := StringReplace(Value, char(222), 'T', [rfReplaceAll]);
  Value := StringReplace(Value, char(206), 'I', [rfReplaceAll]);

  Value := StringReplace(Value, char(227), 'a', [rfReplaceAll]);
  Value := StringReplace(Value, char(226), 'a', [rfReplaceAll]);
  Value := StringReplace(Value, char(186), 's', [rfReplaceAll]);
  Value := StringReplace(Value, char(254), 't', [rfReplaceAll]);
  Value := StringReplace(Value, char(238), 'i', [rfReplaceAll]);
  Result := Value;
End;
//******************* TForm1.ReplaceAllDiacritics *************************

Procedure TForm1.ReplaceAllDiacritics;
Var
  I, j, k, Size     : Integer;
  S                 : String;
Begin
  ProgressBar1.Max := GetAllTableRecCount;
  ProgressBar1.Visible := True;
  EnableControls(False);

  For I := 0 To LstFisiere.Count - 1 Do
    Begin
      If Aborted Then Break;
      StatusBar1.Panels[1].Text := Format('Procesez fisierul %s', [LstFisiere.Strings[I]]);
      Table1.Active := False;
      Table1.TableName := LstFisiere.Strings[I];
      Table1.Active := True;
      k := 0;
      Size := Table1.RecordCount;
      While Not (Table1.Eof Or Aborted) Do
        Begin
          Table1.Edit;
          For j := 0 To Table1.FieldCount - 1 Do
            Table1.Fields[j].AsString := ReplaceDiacritice(Table1.Fields[j].AsString);
          Table1.Post;
          Table1.Next;
          Inc(k);
          StatusBar1.Panels[2].Text := Format('%d / %d', [k, Size]);
          ProgressBar1.StepIt;
          Application.ProcessMessages;
        End;
    End;
  EnableControls(True);
  ProgressBar1.Visible := False;
End;
//******************* TForm1.Button1Click *************************

Procedure TForm1.Button1Click(Sender: TObject);
Begin
  CreazaLista;
  ReplaceAllDiacritics;
End;

End.

